@@ -297,7 +297,6 @@ void parse_dnssl (const struct nd_opt_hdr *opt, unsigned int ifindex)
 	size_t nd_opt_len = opt->nd_opt_len;
 	uint64_t lifetime;
 	char *dom_list;
-	size_t last_dom_len = 0;
 
 	dnssl_opt = (struct nd_opt_dnssl *) opt;
 
@@ -386,7 +385,8 @@ int parse_nd_opts (const struct nd_opt_hdr *opt, size_t opts_len, unsigned int i
 	     opt = (const struct nd_opt_hdr *)
 		   ((const uint8_t *) opt + (opt->nd_opt_len << 3))) {
 
-		if (opt->nd_opt_len == 0 || opts_len < (opt->nd_opt_len << 3))
+		if (opt->nd_opt_len == 0
+		 || opts_len < ((size_t)opt->nd_opt_len << 3))
 			return -1;
 
 		switch (opt->nd_opt_type)
